// ***************************************************************************
// ***************************************************************************
// Copyright 2011(c) Analog Devices, Inc.
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//     - Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     - Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     - Neither the name of Analog Devices, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//     - The use of this software may or may not infringe the patent rights
//       of one or more patent holders.  This license does not release you
//       from the requirement that you obtain separate licenses from these
//       patent holders to use this software.
//     - Use of the software either in source or binary form, must be run
//       on or directly connected to an Analog Devices Inc. component.
//    
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED.
//
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
// RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ***************************************************************************
// ***************************************************************************
// Rejeesh.Kutty@Analog.com (c) Analog Devices Inc.
// ***************************************************************************
// ***************************************************************************

`timescale 1ns/100ps

module cf_adc_1c (

  adc_clk_in_p,
  adc_clk_in_n,
  adc_data_in_p,
  adc_data_in_n,
  adc_data_or_p,
  adc_data_or_n,

  spi_cs0n,
  spi_cs1n,
  spi_clk,
  spi_sd_en,
  spi_sd_o,
  spi_sd_i,

  dma_clk,
  dma_valid,
  dma_data,
  dma_be,
  dma_last,
  dma_ready,

  up_rstn,
  up_clk,
  up_sel,
  up_rwn,
  up_addr,
  up_wdata,
  up_rdata,
  up_ack,
  up_status,

  delay_rst,
  delay_clk,

  debug_clk,
  debug_data,
  debug_trigger);

  parameter C_CF_BUFTYPE = 0;

  input           adc_clk_in_p;
  input           adc_clk_in_n;
  input   [ 7:0]  adc_data_in_p;
  input   [ 7:0]  adc_data_in_n;
  input           adc_data_or_p;
  input           adc_data_or_n;

  output          spi_cs0n;
  output          spi_cs1n;
  output          spi_clk;
  output          spi_sd_en;
  output          spi_sd_o;
  input           spi_sd_i;

  input           dma_clk;
  output          dma_valid;
  output  [63:0]  dma_data;
  output  [ 7:0]  dma_be;
  output          dma_last;
  input           dma_ready;

  input           up_rstn;
  input           up_clk;
  input           up_sel;
  input           up_rwn;
  input   [ 4:0]  up_addr;
  input   [31:0]  up_wdata;
  output  [31:0]  up_rdata;
  output          up_ack;
  output  [ 7:0]  up_status;

  input           delay_rst;
  input           delay_clk;

  output          debug_clk;
  output  [63:0]  debug_data;
  output  [ 7:0]  debug_trigger;

  reg             up_spi_start;
  reg             up_spi_devsel;
  reg     [23:0]  up_spi_wdata;
  reg             up_capture;
  reg     [15:0]  up_capture_count;
  reg             up_dma_unf_hold;
  reg             up_dma_ovf_hold;
  reg             up_dma_status;
  reg             up_adc_or_hold;
  reg             up_adc_pn_oos_hold;
  reg             up_adc_pn_err_hold;
  reg             up_pn_type;
  reg             up_data_sel;
  reg     [ 7:0]  up_status;
  reg     [31:0]  up_rdata;
  reg             up_sel_d;
  reg             up_sel_2d;
  reg             up_ack;
  reg             up_dma_ovf_m1;
  reg             up_dma_ovf_m2;
  reg             up_dma_ovf;
  reg             up_dma_unf_m1;
  reg             up_dma_unf_m2;
  reg             up_dma_unf;
  reg             up_dma_complete_m1;
  reg             up_dma_complete_m2;
  reg             up_dma_complete_m3;
  reg             up_dma_complete;
  reg             up_adc_or_m1;
  reg             up_adc_or_m2;
  reg             up_adc_or;
  reg             up_adc_pn_oos_m1;
  reg             up_adc_pn_oos_m2;
  reg             up_adc_pn_oos;
  reg             up_adc_pn_err_m1;
  reg             up_adc_pn_err_m2;
  reg             up_adc_pn_err;
  reg             up_adc_fs_toggle_m1;
  reg             up_adc_fs_toggle_m2;
  reg             up_adc_fs_toggle_m3;
  reg     [31:0]  up_adc_fs_data;

  wire            up_wr_s;
  wire            up_rd_s;
  wire            up_ack_s;
  wire            up_adc_fs_toggle_s;
  wire    [ 7:0]  up_spi_rdata_s;
  wire            up_spi_status_s;
  wire            dma_ovf_s;
  wire            dma_unf_s;
  wire            dma_complete_s;
  wire            adc_wr_s;
  wire    [ 5:0]  adc_waddr_s;
  wire    [64:0]  adc_wdata_s;
  wire            adc_fs_toggle_s;
  wire    [ 5:0]  adc_fs_waddr_s;
  wire    [31:0]  adc_fs_data_s;
  wire            adc_rel_toggle_s;
  wire    [ 5:0]  adc_rel_waddr_s;
  wire    [ 5:0]  adc_waddr_g_s;
  wire            adc_or_s;
  wire            adc_pn_oos_s;
  wire            adc_pn_err_s;
  wire    [63:0]  spi_debug_data_s;
  wire    [ 7:0]  spi_debug_trigger_s;
  wire    [63:0]  dma_debug_data_s;
  wire    [ 7:0]  dma_debug_trigger_s;
  wire    [63:0]  adc_debug_data_s;
  wire    [ 7:0]  adc_debug_trigger_s;

  wire            adc_clk;

  assign debug_clk = adc_clk;
  assign debug_data = adc_debug_data_s;
  assign debug_trigger = adc_debug_trigger_s;

  assign up_wr_s = up_sel & ~up_rwn;
  assign up_rd_s = up_sel & up_rwn;
  assign up_ack_s = up_sel_d & ~up_sel_2d;

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_spi_start <= 'd0;
      up_spi_devsel <= 'd0;
      up_spi_wdata <= 'd0;
      up_capture <= 'd0;
      up_capture_count <= 'd0;
      up_dma_unf_hold <= 'd0;
      up_dma_ovf_hold <= 'd0;
      up_dma_status <= 'd0;
      up_adc_or_hold <= 'd0;
      up_adc_pn_oos_hold <= 'd0;
      up_adc_pn_err_hold <= 'd0;
      up_pn_type <= 'd0;
      up_data_sel <= 'd0;
      up_status <= 'd0;
    end else begin
      if ((up_addr == 5'h01) && (up_wr_s == 1'b1)) begin
        up_spi_start <= up_wdata[25];
        up_spi_devsel <= up_wdata[24];
        up_spi_wdata <= up_wdata[23:0];
      end
      if ((up_addr == 5'h03) && (up_wr_s == 1'b1)) begin
        up_capture <= up_wdata[16];
        up_capture_count <= up_wdata[15:0];
      end
      if (up_dma_unf == 1'b1) begin
        up_dma_unf_hold <= 1'b1;
      end else if ((up_addr == 5'h04) && (up_wr_s == 1'b1)) begin
        up_dma_unf_hold <= up_dma_unf_hold & ~up_wdata[2];
      end
      if (up_dma_ovf == 1'b1) begin
        up_dma_ovf_hold <= 1'b1;
      end else if ((up_addr == 5'h04) && (up_wr_s == 1'b1)) begin
        up_dma_ovf_hold <= up_dma_ovf_hold & ~up_wdata[2];
      end
      if (up_dma_complete == 1'b1) begin
        up_dma_status <= 1'b0;
      end else if ((up_addr == 5'h03) && (up_wr_s == 1'b1) && (up_dma_status == 1'b0)) begin
        up_dma_status <= up_wdata[16];
      end
      if (up_adc_or == 1'b1) begin
        up_adc_or_hold <= 1'b1;
      end else if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_adc_or_hold <= up_adc_or_hold & ~up_wdata[0];
      end
      if (up_adc_pn_oos == 1'b1) begin
        up_adc_pn_oos_hold <= 1'b1;
      end else if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_adc_pn_oos_hold <= up_adc_pn_oos_hold & ~up_wdata[1];
      end
      if (up_adc_pn_err == 1'b1) begin
        up_adc_pn_err_hold <= 1'b1;
      end else if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_adc_pn_err_hold <= up_adc_pn_err_hold & ~up_wdata[2];
      end
      if ((up_addr == 5'h09) && (up_wr_s == 1'b1)) begin
        up_pn_type <= up_wdata[0];
      end
      if ((up_addr == 5'h0a) && (up_wr_s == 1'b1)) begin
        up_data_sel <= up_wdata[0];
      end
      up_status <= {1'b1, up_capture, up_dma_ovf, up_dma_status,
        up_spi_devsel, up_spi_status_s, 1'b0, up_data_sel};
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_rdata <= 'd0;
      up_sel_d <= 'd0;
      up_sel_2d <= 'd0;
      up_ack <= 'd0;
    end else begin
      case (up_addr)
        5'h00: up_rdata <= 32'h00010061;
        5'h01: up_rdata <= {6'd0, up_spi_start, up_spi_devsel, up_spi_wdata};
        5'h02: up_rdata <= {23'd0, up_spi_status_s, up_spi_rdata_s};
        5'h03: up_rdata <= {15'd0, up_capture, up_capture_count};
        5'h04: up_rdata <= {29'd0, up_dma_unf_hold, up_dma_ovf_hold, up_dma_status};
        5'h05: up_rdata <= {29'd0, up_adc_pn_err_hold, up_adc_pn_oos_hold, up_adc_or_hold};
        5'h06: up_rdata <= up_adc_fs_data;
        5'h09: up_rdata <= {31'd0, up_pn_type};
        5'h0a: up_rdata <= {31'd0, up_data_sel};
        default: up_rdata <= 0;
      endcase
      up_sel_d <= up_sel;
      up_sel_2d <= up_sel_d;
      up_ack <= up_ack_s;
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_dma_ovf_m1 <= 'd0;
      up_dma_ovf_m2 <= 'd0;
      up_dma_ovf <= 'd0;
      up_dma_unf_m1 <= 'd0;
      up_dma_unf_m2 <= 'd0;
      up_dma_unf <= 'd0;
      up_dma_complete_m1 <= 'd0;
      up_dma_complete_m2 <= 'd0;
      up_dma_complete_m3 <= 'd0;
      up_dma_complete <= 'd0;
    end else begin
      up_dma_ovf_m1 <= dma_ovf_s;
      up_dma_ovf_m2 <= up_dma_ovf_m1;
      up_dma_ovf <= up_dma_ovf_m2;
      up_dma_unf_m1 <= dma_unf_s;
      up_dma_unf_m2 <= up_dma_unf_m1;
      up_dma_unf <= up_dma_unf_m2;
      up_dma_complete_m1 <= dma_complete_s;
      up_dma_complete_m2 <= up_dma_complete_m1;
      up_dma_complete_m3 <= up_dma_complete_m2;
      up_dma_complete <= up_dma_complete_m3 ^ up_dma_complete_m2;
    end
  end

  assign up_adc_fs_toggle_s = up_adc_fs_toggle_m3 ^ up_adc_fs_toggle_m2;

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_adc_or_m1 <= 'd0;
      up_adc_or_m2 <= 'd0;
      up_adc_or <= 'd0;
      up_adc_pn_oos_m1 <= 'd0;
      up_adc_pn_oos_m2 <= 'd0;
      up_adc_pn_oos <= 'd0;
      up_adc_pn_err_m1 <= 'd0;
      up_adc_pn_err_m2 <= 'd0;
      up_adc_pn_err <= 'd0;
      up_adc_fs_toggle_m1 <= 'd0;
      up_adc_fs_toggle_m2 <= 'd0;
      up_adc_fs_toggle_m3 <= 'd0;
      up_adc_fs_data <= 'd0;
    end else begin
      up_adc_or_m1 <= adc_or_s;
      up_adc_or_m2 <= up_adc_or_m1;
      up_adc_or <= up_adc_or_m2;
      up_adc_pn_oos_m1 <= adc_pn_oos_s;
      up_adc_pn_oos_m2 <= up_adc_pn_oos_m1;
      up_adc_pn_oos <= up_adc_pn_oos_m2;
      up_adc_pn_err_m1 <= adc_pn_err_s;
      up_adc_pn_err_m2 <= up_adc_pn_err_m1;
      up_adc_pn_err <= up_adc_pn_err_m2;
      up_adc_fs_toggle_m1 <= adc_fs_toggle_s;
      up_adc_fs_toggle_m2 <= up_adc_fs_toggle_m1;
      up_adc_fs_toggle_m3 <= up_adc_fs_toggle_m2;
      if (up_adc_fs_toggle_s == 1'b1) begin
        up_adc_fs_data <= adc_fs_data_s;
      end
    end
  end

  cf_spi i_spi (
    .spi_cs0n (spi_cs0n),
    .spi_cs1n (spi_cs1n),
    .spi_clk (spi_clk),
    .spi_sd_en (spi_sd_en),
    .spi_sd_o (spi_sd_o),
    .spi_sd_i (spi_sd_i),
    .up_rstn (up_rstn),
    .up_clk (up_clk),
    .up_spi_start (up_spi_start),
    .up_spi_devsel (up_spi_devsel),
    .up_spi_wdata (up_spi_wdata),
    .up_spi_rdata (up_spi_rdata_s),
    .up_spi_status (up_spi_status_s),
    .debug_data (spi_debug_data_s),
    .debug_trigger (spi_debug_trigger_s));

  cf_dma_wr i_dma_wr (
    .adc_clk (adc_clk),
    .adc_wr (adc_wr_s),
    .adc_waddr (adc_waddr_s),
    .adc_wdata (adc_wdata_s),
    .adc_fs_toggle (adc_fs_toggle_s),
    .adc_fs_waddr (adc_fs_waddr_s),
    .adc_rel_toggle (adc_rel_toggle_s),
    .adc_rel_waddr (adc_rel_waddr_s),
    .adc_waddr_g (adc_waddr_g_s),
    .dma_clk (dma_clk),
    .dma_valid (dma_valid),
    .dma_data (dma_data),
    .dma_be (dma_be),
    .dma_last (dma_last),
    .dma_ready (dma_ready),
    .dma_ovf (dma_ovf_s),
    .dma_unf (dma_unf_s),
    .dma_complete (dma_complete_s),
    .debug_data (dma_debug_data_s),
    .debug_trigger (dma_debug_trigger_s));

  cf_adc_if i_adc_if (
    .adc_clk_in_p (adc_clk_in_p),
    .adc_clk_in_n (adc_clk_in_n),
    .adc_data_in_p (adc_data_in_p),
    .adc_data_in_n (adc_data_in_n),
    .adc_data_or_p (adc_data_or_p),
    .adc_data_or_n (adc_data_or_n),
    .adc_clk (adc_clk),
    .adc_wr (adc_wr_s),
    .adc_waddr (adc_waddr_s),
    .adc_wdata (adc_wdata_s),
    .adc_fs_toggle (adc_fs_toggle_s),
    .adc_fs_waddr (adc_fs_waddr_s),
    .adc_fs_data (adc_fs_data_s),
    .adc_rel_toggle (adc_rel_toggle_s),
    .adc_rel_waddr (adc_rel_waddr_s),
    .adc_waddr_g (adc_waddr_g_s),
    .adc_or (adc_or_s),
    .adc_pn_oos (adc_pn_oos_s),
    .adc_pn_err (adc_pn_err_s),
    .up_capture (up_capture),
    .up_capture_count (up_capture_count),
    .up_pn_type (up_pn_type),
    .up_data_sel (up_data_sel),
    .debug_data (adc_debug_data_s),
    .debug_trigger (adc_debug_trigger_s));

  defparam i_adc_if.C_CF_BUFTYPE = C_CF_BUFTYPE;

endmodule

// ***************************************************************************
// ***************************************************************************
