// ***************************************************************************
// ***************************************************************************
// Copyright 2011(c) Analog Devices, Inc.
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//     - Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     - Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     - Neither the name of Analog Devices, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//     - The use of this software may or may not infringe the patent rights
//       of one or more patent holders.  This license does not release you
//       from the requirement that you obtain separate licenses from these
//       patent holders to use this software.
//     - Use of the software either in source or binary form, must be run
//       on or directly connected to an Analog Devices Inc. component.
//    
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED.
//
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
// RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ***************************************************************************
// ***************************************************************************
// Rejeesh.Kutty@Analog.com (c) Analog Devices Inc.
// ***************************************************************************
// ***************************************************************************
// This is the LVDS/DDR interface

`timescale 1ns/100ps

module cf_adc_if (

  adc_clk_in_p,
  adc_clk_in_n,
  adc_data_in_p,
  adc_data_in_n,
  adc_data_or_p,
  adc_data_or_n,

  adc_clk,
  adc_wr,
  adc_waddr,
  adc_wdata,
  adc_fs_toggle,
  adc_fs_waddr,
  adc_fs_data,
  adc_rel_toggle,
  adc_rel_waddr,
  adc_waddr_g,

  adc_or,
  adc_pn_oos,
  adc_pn_err,

  up_capture,
  up_capture_count,
  up_pn_type,
  up_data_sel,

  debug_data,
  debug_trigger);

  parameter C_CF_BUFTYPE = 0;

  input           adc_clk_in_p;
  input           adc_clk_in_n;
  input   [ 7:0]  adc_data_in_p;
  input   [ 7:0]  adc_data_in_n;
  input           adc_data_or_p;
  input           adc_data_or_n;

  output          adc_clk;
  output          adc_wr;
  output  [ 5:0]  adc_waddr;
  output  [64:0]  adc_wdata;
  output          adc_fs_toggle;
  output  [ 5:0]  adc_fs_waddr;
  output  [31:0]  adc_fs_data;
  output          adc_rel_toggle;
  output  [ 5:0]  adc_rel_waddr;
  output  [ 5:0]  adc_waddr_g;

  output          adc_or;
  output          adc_pn_oos;
  output          adc_pn_err;


  input           up_capture;
  input   [15:0]  up_capture_count;
  input           up_pn_type;
  input           up_data_sel;

  output  [63:0]  debug_data;
  output  [ 7:0]  debug_trigger;

  reg             adc_up_capture_m1 = 'd0;
  reg             adc_up_capture_m2 = 'd0;
  reg             adc_up_capture_m3 = 'd0;
  reg             adc_up_capture = 'd0;
  reg     [18:0]  adc_capture_count = 'd0;
  reg             adc_capture = 'd0;
  reg             adc_capture_d = 'd0;
  reg             adc_capture_last = 'd0;
  reg             adc_pn_type_m1 = 'd0;
  reg             adc_pn_type_m2 = 'd0;
  reg             adc_pn_type = 'd0;
  reg             adc_data_sel_m1 = 'd0;
  reg             adc_data_sel_m2 = 'd0;
  reg             adc_data_sel = 'd0;
  reg             adc_fs_toggle = 'd0;
  reg     [ 5:0]  adc_fs_waddr = 'd0;
  reg     [31:0]  adc_fs_data = 'd0;
  reg             adc_rel_toggle = 'd0;
  reg     [ 5:0]  adc_rel_waddr = 'd0;
  reg     [15:0]  adc_data_16_1d = 'd0;
  reg     [15:0]  adc_data_16_2d = 'd0;
  reg     [15:0]  adc_data_16_3d = 'd0;
  reg             adc_wr = 'd0;
  reg     [ 7:0]  adc_waddr_8 = 'd0;
  reg     [ 5:0]  adc_waddr_g = 'd0;
  reg             adc_pn_en = 'd0;
  reg     [31:0]  adc_pn_data = 'd0;
  reg             adc_pn_en_d = 'd0;
  reg             adc_pn_match = 'd0;
  reg     [ 6:0]  adc_pn_oos_count = 'd0;
  reg             adc_pn_oos = 'd0;
  reg     [ 4:0]  adc_pn_err_count = 'd0;
  reg             adc_pn_err = 'd0;
  reg     [ 4:0]  adc_or_count = 'd0;
  reg             adc_or = 'd0;
  reg     [ 7:0]  adc_data_q1 = 'd0;
  reg     [ 7:0]  adc_data_q2 = 'd0;
  reg     [ 7:0]  adc_data_q2_d = 'd0;
  reg     [ 7:0]  adc_data_p = 'd0;
  reg     [ 7:0]  adc_data_n = 'd0;
  reg     [15:0]  adc_data_16 = 'd0;
  reg             adc_or_q1 = 'd0;
  reg             adc_or_q2 = 'd0;
  reg             adc_or_q2_d = 'd0;
  reg             adc_or_p = 'd0;
  reg             adc_or_n = 'd0;
  reg             adc_or_16 = 'd0;

  wire            adc_capture_start_s;
  wire            adc_capture_end_s;
  wire            adc_capture_4_s;
  wire            adc_capture_last_s;
  wire    [31:0]  adc_pn_data_in_s;
  wire            adc_pn_match_s;
  wire    [31:0]  adc_pn_data_s;
  wire            adc_pn_err_s;
  wire    [ 7:0]  adc_data_ibuf_s;
  wire    [ 7:0]  adc_data_q1_s;
  wire    [ 7:0]  adc_data_q2_s;
  wire            adc_or_ibuf_s;
  wire            adc_or_q1_s;
  wire            adc_or_q2_s;
  wire            adc_clk_ibuf_s;

  genvar          l_inst;

  // pn equations

  function [31:0] pn9;
    input [31:0] din;
    reg   [31:0] dout;
    begin
      dout[31] = din[8] ^ din[4];
      dout[30] = din[7] ^ din[3];
      dout[29] = din[6] ^ din[2];
      dout[28] = din[5] ^ din[1];
      dout[27] = din[4] ^ din[0];
      dout[26] = din[3] ^ din[8] ^ din[4];
      dout[25] = din[2] ^ din[7] ^ din[3];
      dout[24] = din[1] ^ din[6] ^ din[2];
      dout[23] = din[0] ^ din[5] ^ din[1];
      dout[22] = din[8] ^ din[0];
      dout[21] = din[7] ^ din[8] ^ din[4];
      dout[20] = din[6] ^ din[7] ^ din[3];
      dout[19] = din[5] ^ din[6] ^ din[2];
      dout[18] = din[4] ^ din[5] ^ din[1];
      dout[17] = din[3] ^ din[4] ^ din[0];
      dout[16] = din[2] ^ din[3] ^ din[8] ^ din[4];
      dout[15] = din[1] ^ din[2] ^ din[7] ^ din[3];
      dout[14] = din[0] ^ din[1] ^ din[6] ^ din[2];
      dout[13] = din[8] ^ din[0] ^ din[4] ^ din[5] ^ din[1];
      dout[12] = din[7] ^ din[8] ^ din[3] ^ din[0];
      dout[11] = din[6] ^ din[7] ^ din[2] ^ din[8] ^ din[4];
      dout[10] = din[5] ^ din[6] ^ din[1] ^ din[7] ^ din[3];
      dout[ 9] = din[4] ^ din[5] ^ din[0] ^ din[6] ^ din[2];
      dout[ 8] = din[3] ^ din[8] ^ din[5] ^ din[1];
      dout[ 7] = din[2] ^ din[4] ^ din[7] ^ din[0];
      dout[ 6] = din[1] ^ din[3] ^ din[6] ^ din[8] ^ din[4];
      dout[ 5] = din[0] ^ din[2] ^ din[5] ^ din[7] ^ din[3];
      dout[ 4] = din[8] ^ din[1] ^ din[6] ^ din[2];
      dout[ 3] = din[7] ^ din[0] ^ din[5] ^ din[1];
      dout[ 2] = din[6] ^ din[8] ^ din[0];
      dout[ 1] = din[5] ^ din[7] ^ din[8] ^ din[4];
      dout[ 0] = din[4] ^ din[6] ^ din[7] ^ din[3];
      pn9 = dout;
    end
  endfunction

  function [31:0] pn23;
    input [31:0] din;
    reg   [31:0] dout;
    begin
      dout[31] = din[22] ^ din[17];
      dout[30] = din[21] ^ din[16];
      dout[29] = din[20] ^ din[15];
      dout[28] = din[19] ^ din[14];
      dout[27] = din[18] ^ din[13];
      dout[26] = din[17] ^ din[12];
      dout[25] = din[16] ^ din[11];
      dout[24] = din[15] ^ din[10];
      dout[23] = din[14] ^ din[ 9];
      dout[22] = din[13] ^ din[ 8];
      dout[21] = din[12] ^ din[ 7];
      dout[20] = din[11] ^ din[ 6];
      dout[19] = din[10] ^ din[ 5];
      dout[18] = din[ 9] ^ din[ 4];
      dout[17] = din[ 8] ^ din[ 3];
      dout[16] = din[ 7] ^ din[ 2];
      dout[15] = din[ 6] ^ din[ 1];
      dout[14] = din[ 5] ^ din[ 0];
      dout[13] = din[ 4] ^ din[22] ^ din[17];
      dout[12] = din[ 3] ^ din[21] ^ din[16];
      dout[11] = din[ 2] ^ din[20] ^ din[15];
      dout[10] = din[ 1] ^ din[19] ^ din[14];
      dout[ 9] = din[ 0] ^ din[18] ^ din[13];
      dout[ 8] = din[22] ^ din[12];
      dout[ 7] = din[21] ^ din[11];
      dout[ 6] = din[20] ^ din[10];
      dout[ 5] = din[19] ^ din[ 9];
      dout[ 4] = din[18] ^ din[ 8];
      dout[ 3] = din[17] ^ din[ 7];
      dout[ 2] = din[16] ^ din[ 6];
      dout[ 1] = din[15] ^ din[ 5];
      dout[ 0] = din[14] ^ din[ 4];
      pn23 = dout;
    end
  endfunction

  // binary to grey conversion

  function [5:0] b2g;
    input [5:0] b;
    reg   [5:0] g;
    begin
      g[5] = b[5];
      g[4] = b[5] ^ b[4];
      g[3] = b[4] ^ b[3];
      g[2] = b[3] ^ b[2];
      g[1] = b[2] ^ b[1];
      g[0] = b[1] ^ b[0];
      b2g = g;
    end
  endfunction

  assign debug_trigger[7] = 'd0;
  assign debug_trigger[6] = adc_capture_last_s;
  assign debug_trigger[5] = adc_fs_toggle;
  assign debug_trigger[4] = (adc_data_n == 'hff) ? 1'b1 : 1'b0;
  assign debug_trigger[3] = (adc_data_p == 'hff) ? 1'b1 : 1'b0;
  assign debug_trigger[2] = adc_capture_end_s;
  assign debug_trigger[1] = adc_capture_start_s;
  assign debug_trigger[0] = (adc_data_16 == 'd0) ? 'd0 : 'd1;
  
  assign debug_data[63:60] = adc_waddr[3:0];
  assign debug_data[59:59] = adc_capture_count[18];
  assign debug_data[58:52] = adc_capture_count[6:0];
  assign debug_data[51:51] = adc_capture;
  assign debug_data[50:50] = adc_capture_4_s;
  assign debug_data[49:49] = adc_capture_end_s;
  assign debug_data[48:48] = adc_capture_start_s;
  assign debug_data[47:40] = adc_data_q2_s;
  assign debug_data[39:32] = adc_data_q1_s;
  assign debug_data[31:24] = adc_data_n;
  assign debug_data[23:16] = adc_data_p;
  assign debug_data[15: 0] = adc_data_16;

  // adc capture control

  assign adc_capture_start_s = adc_capture_count[18] & ~adc_capture;
  assign adc_capture_end_s = adc_capture_d & ~adc_capture;
  assign adc_capture_4_s = (adc_waddr_8[3:0] == 4'b1100) ? adc_wr : 1'b0;
  assign adc_capture_last_s = (adc_capture_count[17:2] == 'd0) ? 1'b1 : 1'b0;

  always @(posedge adc_clk) begin
    adc_up_capture_m1 <= up_capture;
    adc_up_capture_m2 <= adc_up_capture_m1;
    adc_up_capture_m3 <= adc_up_capture_m2;
    adc_up_capture <= adc_up_capture_m2 & ~adc_up_capture_m3;
    if (adc_capture_count[18] == 1'b1) begin
      adc_capture_count <= adc_capture_count - 1'b1;
    end else if (adc_up_capture == 1'b1) begin
      adc_capture_count <= {1'd1, up_capture_count, 2'd3};
    end
    adc_capture <= adc_capture_count[18];
    adc_capture_d <= adc_capture;
    adc_capture_last <= adc_capture_last_s;
  end

  always @(posedge adc_clk) begin
    adc_pn_type_m1 <= up_pn_type;
    adc_pn_type_m2 <= adc_pn_type_m1;
    adc_pn_type <= adc_pn_type_m2;
    adc_data_sel_m1 <= up_data_sel;
    adc_data_sel_m2 <= adc_data_sel_m1;
    adc_data_sel <= adc_data_sel_m2;
  end

  // adc data write

  always @(posedge adc_clk) begin
    if (adc_capture_start_s == 1'b1) begin
      adc_fs_toggle <= ~adc_fs_toggle;
      adc_fs_waddr <= adc_waddr;
      adc_fs_data <= adc_wdata[63:32];
    end
    if ((adc_capture_4_s == 1'b1) || (adc_capture_end_s == 1'b1)) begin
      adc_rel_toggle <= ~adc_rel_toggle;
      adc_rel_waddr <= adc_waddr;
    end
  end

  assign adc_waddr = adc_waddr_8[7:2];
  assign adc_wdata = {adc_capture_last, adc_data_16, adc_data_16_1d,
    adc_data_16_2d, adc_data_16_3d};

  always @(posedge adc_clk) begin
    adc_data_16_1d <= adc_data_16;
    adc_data_16_2d <= adc_data_16_1d;
    adc_data_16_3d <= adc_data_16_2d;
    adc_wr <= adc_capture_count[18];
    if (adc_wr == 1'b1) begin
      adc_waddr_8 <= adc_waddr_8 + 1'b1;
    end
    adc_waddr_g <= b2g(adc_waddr_8[7:2]);
  end

  // prbs pattern check

  assign adc_pn_data_in_s = {~adc_data_16_1d[15], adc_data_16_1d[14:0],
    ~adc_data_16[15], adc_data_16[14:0]};
  assign adc_pn_match_s = (adc_pn_data_in_s == adc_pn_data) ? 1'b1 : 1'b0;
  assign adc_pn_data_s = (adc_pn_oos == 1'b1) ? adc_pn_data_in_s : adc_pn_data;

  always @(posedge adc_clk) begin
    adc_pn_en <= ~adc_pn_en;
    if (adc_pn_en == 1'b1) begin
      if (adc_pn_type == 1'b0) begin
        adc_pn_data <= pn9(adc_pn_data_s);
      end else begin
        adc_pn_data <= pn23(adc_pn_data_s);
      end
    end
  end

  always @(posedge adc_clk) begin
    adc_pn_en_d <= adc_pn_en;
    adc_pn_match <= adc_pn_match_s;
    if (adc_pn_en_d == 1'b1) begin
      if (adc_pn_oos == 1'b1) begin
        if (adc_pn_match == 1'b1) begin
          if (adc_pn_oos_count >= 16) begin
            adc_pn_oos_count <= 'd0;
            adc_pn_oos <= 'd0;
          end else begin
            adc_pn_oos_count <= adc_pn_oos_count + 1'b1;
            adc_pn_oos <= 'd1;
          end
        end else begin
          adc_pn_oos_count <= 'd0;
          adc_pn_oos <= 'd1;
        end
      end else begin
        if (adc_pn_match == 1'b0) begin
          if (adc_pn_oos_count >= 64) begin
            adc_pn_oos_count <= 'd0;
            adc_pn_oos <= 'd1;
          end else begin
            adc_pn_oos_count <= adc_pn_oos_count + 1'b1;
            adc_pn_oos <= 'd0;
          end
        end else begin
          adc_pn_oos_count <= 'd0;
          adc_pn_oos <= 'd0;
        end
      end
    end
  end

  assign adc_pn_err_s = ~(adc_pn_oos | adc_pn_match);
    
  always @(posedge adc_clk) begin
    if (adc_pn_en_d == 1'b1) begin
      if (adc_pn_err_s == 1'b1) begin
        adc_pn_err_count <= 5'h10;
      end else if (adc_pn_err_count[4] == 1'b1) begin
        adc_pn_err_count <= adc_pn_err_count + 1'b1;
      end
    end
    adc_pn_err <= adc_pn_err_count[4];
  end

  // over the range is independent of data path

  always @(posedge adc_clk) begin
    if (adc_or_16 == 1'b1) begin
      adc_or_count <= 5'h10;
    end else if (adc_or_count[4] == 1'b1) begin
      adc_or_count <= adc_or_count + 1'b1;
    end
    adc_or <= adc_or_count[4];
  end

  // rearrange data from adc

  always @(posedge adc_clk) begin
    adc_data_q1 <= adc_data_q1_s;
    adc_data_q2 <= adc_data_q2_s;
    adc_data_q2_d <= adc_data_q2;
    adc_data_p <= (adc_data_sel == 1'b1) ? adc_data_q2 : adc_data_q1;
    adc_data_n <= (adc_data_sel == 1'b1) ? adc_data_q1 : adc_data_q2_d;
    adc_data_16[15] <= adc_data_n[7];
    adc_data_16[14] <= adc_data_p[7];
    adc_data_16[13] <= adc_data_n[6];
    adc_data_16[12] <= adc_data_p[6];
    adc_data_16[11] <= adc_data_n[5];
    adc_data_16[10] <= adc_data_p[5];
    adc_data_16[ 9] <= adc_data_n[4];
    adc_data_16[ 8] <= adc_data_p[4];
    adc_data_16[ 7] <= adc_data_n[3];
    adc_data_16[ 6] <= adc_data_p[3];
    adc_data_16[ 5] <= adc_data_n[2];
    adc_data_16[ 4] <= adc_data_p[2];
    adc_data_16[ 3] <= adc_data_n[1];
    adc_data_16[ 2] <= adc_data_p[1];
    adc_data_16[ 1] <= adc_data_n[0];
    adc_data_16[ 0] <= adc_data_p[0];
    adc_or_q1 <= adc_or_q1_s;
    adc_or_q2 <= adc_or_q2_s;
    adc_or_q2_d <= adc_or_q2;
    adc_or_p <= (adc_data_sel == 1'b1) ? adc_or_q2 : adc_or_q1;
    adc_or_n <= (adc_data_sel == 1'b1) ? adc_or_q1 : adc_or_q2_d;
    adc_or_16 <= adc_or_p | adc_or_n;
  end

  generate
  for (l_inst = 0; l_inst <= 7; l_inst = l_inst + 1) begin : g_adc_if

  IBUFDS i_data_ibuf (
    .I (adc_data_in_p[l_inst]),
    .IB (adc_data_in_n[l_inst]),
    .O (adc_data_ibuf_s[l_inst]));

  IDDR #(
    .INIT_Q1 (1'b0),
    .INIT_Q2 (1'b0),
    .DDR_CLK_EDGE ("SAME_EDGE_PIPELINED"),
    .SRTYPE ("ASYNC"))
  i_data_ddr (
    .CE (1'b1),
    .R (1'b0),
    .S (1'b0),
    .C (adc_clk),
    .D (adc_data_ibuf_s[l_inst]),
    .Q1 (adc_data_q1_s[l_inst]),
    .Q2 (adc_data_q2_s[l_inst]));

  end
  endgenerate

  IBUFDS i_data_or_ibuf (
    .I (adc_data_or_p),
    .IB (adc_data_or_n),
    .O (adc_or_ibuf_s));

  IDDR #(
    .INIT_Q1 (1'b0),
    .INIT_Q2 (1'b0),
    .DDR_CLK_EDGE ("SAME_EDGE_PIPELINED"),
    .SRTYPE ("ASYNC"))
  i_or_ddr (
    .CE (1'b1),
    .R (1'b0),
    .S (1'b0),
    .C (adc_clk),
    .D (adc_or_ibuf_s),
    .Q1 (adc_or_q1_s),
    .Q2 (adc_or_q2_s));

  IBUFGDS i_clk_ibuf (
    .I (adc_clk_in_p),
    .IB (adc_clk_in_n),
    .O (adc_clk_ibuf_s));

  generate
  if (C_CF_BUFTYPE == 1) begin
  BUFG i_clk_gbuf (
    .I (adc_clk_ibuf_s),
    .O (adc_clk));
  end else begin
  BUFR #(
    .SIM_DEVICE ("VIRTEX6"),
    .BUFR_DIVIDE ("BYPASS"))
  i_clk_rbuf (
    .CLR (1'b0),
    .CE (1'b1),
    .I (adc_clk_ibuf_s),
    .O (adc_clk));
  end
  endgenerate

endmodule

// ***************************************************************************
// ***************************************************************************

